﻿function Get-WmiEventFilter
{
	[CmdletBinding(SupportsShouldProcess = $false)]
	<#
		.Synopsis
		Retrieves an existing WMI event binding.
		
		.Description
		Retrieves an existing WMI event binding.
		
		.Parameter Namespace
		The namespace in which to retrieve __FilterToConsumerBinding instances from.
		
		.Parameter Name
		The name of the __FilterToConsumerBinding instance to retrieve.
		
		.Parameter ComputerName
		The computer on which to retrieve __FilterToConsumerBinding instances.
	#>
	Param(
		# The name of the WMI event filter to retrieve. The name property is the key on the __FilterToConsumerBinding system WMI class.
		[Parameter(
			  Mandatory   = $false
			, HelpMessage = "Please specify the name of the __FilterToConsumerBinding instance you would like to retrieve."
		)]
		[string]
		${Name}
		,
		# The WMI namespace to retrieve event filters from.
		# TODO: Provide an option to retrieve ALL event filters from ALL namespaces?
		${Namespace}
		,
		[string]
		${ComputerName}
		,
		# TODO: Implement parameter to allow searching of the query text
		[string]
		${QuerySearchString}
	)
	
	begin
	{
		# Translate asterisks (wildcards) to percent signs (WMI wildcards)
		${Name} = ${Name}.Replace("*", "%")
		Get-WmiObject -Namespace ${Namespace} -Query "SELECT * FROM __EventFilter WHERE Name LIKE '${Name}'"
	}
}