﻿# TODO: Test what happens if you delete an __EventFilter instance while the __FilterToConsumerBinding instance still exists

function Remove-WmiFilterToConsumerBinding
{
	<#
		.Synopsis
		Removes a binding between a WMI event filter and a WMI event consumer.
		
		.Description
		Removes a binding between a WMI event filter and a WMI event consumer. This function requires the input of a partial filter name and consumer name. These two parameters will be used to identify the binding instance to be removed, because __FilterToConsumerBinding does not provider a friendly "Name" property to identify them by.
		
		.Parameter FilterName
		Partial name of the event filter that the binding is using.
		
		.Parameter ConsumerName
		Partial name of the event consumer that the binding is using.
		
		.Component
		Windows Management Instrumentation (WMI)
		
		.Link
		http://trevorsullivan.net
		
		.Link
		http://powershell.artofshell.com
	#>
	
	param(
		[Parameter(
			  Mandatory   = $false
			, HelpMessage = 'Please enter the WMI namespace where the __FilterToConsumerBinding instance exists.'
		)]
		[string]
		$Namespace = 'root\subscription'
		,
		[Parameter(
			  Mandatory   = $false
			, HelpMessage = 'Please enter the WMI namespace where the __FilterToConsumerBinding instance exists.'
		)]
		[string]
		${FilterName}
		,
		[Parameter(
			  Mandatory   = $false
			, HelpMessage = 'Please enter the WMI namespace where the __FilterToConsumerBinding instance exists.'
		)]
		[string]
		${ConsumerName}
		
	)
	
	Begin
	{
		# Replace wildcards (*) with WMI wildcards (%)
		if (${FilterName})
		{
			${FilterName} = ${FilterName}.Replace('*','%')
		}
		
		# Replace wildcards (*) with WMI wildcards (%)
		if (${ConsumerName})
		{
			${ConsumerName} = ${ConsumerName}.Replace('*','%')
		}
	}
	
	Process
	{
		Get-WmiObject -Query 
	}
	
	End
	{
	}
}

# Export the advanced function
Export-ModuleMember -Function Remove-WmiFilterToConsumerBinding

# Create an alias for the advanced function and export it
Set-Alias -Name rmwmibinding -Value Remove-WmiFilterToConsumerBinding
Export-ModuleMember -Alias rmwmibinding